/*
 * EXAMINATOR -- Create your multiple choice exams on the fly.
 * Copyright (C) 2010 Herbert Haas
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, see http://www.gnu.org/licenses/gpl-2.0.html
 *
 */

#include "examinator.h"


int add_collection (char* cf)
{
	struct collection_struct *c;
	char line[MAX_LINE_LEN], t[2][MAX_TOKEN_LEN];
	int qmode=0, amode=0, len, ret=0, i, qc, ac, n=0;
	FILE *fp=NULL;
		
	if (verbose) fprintf(stderr, "Add collection '%s'\n", cf);

	if ((fp = fopen(cf, "r")) == NULL) {
		fprintf(stderr, "ERROR: Cannot open '%s'!\n", cf);
		ret=1; goto here;
	}

	// Create first list element
	if ((c = create_clist_entry ())==NULL) {
		fprintf(stderr, "ERROR: Cannot reserve memory!\n");
		ret=1; goto here; // should never happen
	}
	
	i=0; qc=0; ac=0; // line-, question-, and answer-count

	while (fgets(line, MAX_LINE_LEN-1, fp) != NULL) {
		i++;
		// Empty lines separate questions and answer blocks:
		if (line_is_empty(line)) {
			if (qmode) amode=1; // answers must follow
			if ((!qmode) && (amode)) {
				amode=0; // end of answers, new question must follow
				qc++;
				if ((c = create_clist_entry ())==NULL) { // make new list element
					fprintf(stderr, "ERROR: Cannot reserve memory!\n");
					ret=1; goto here; // should never happen
				}
			}
			continue;
		}
		
		// question section starts 
		if (!amode) { 
			qmode=1;
			if (line[0]=='@') { // picture reference
				n = sscanf(line, "@%s %s", t[0], t[1]);
				if ((n==0) || (n>2)) {
					fprintf(stderr, "ERROR: Invalid picture parameters in file '%s' line %i!\n",cf,i);
					ret=1; goto here;;
				}
				safe_strncpy(c->picture, t[0], MAX_TOKEN_LEN);
				if (n==2) {
					c->picture_size = (int) str2int(t[1]);
				} else c->picture_size = DEFAULT_PICTURE_SIZE;
			}
			else {	// question lines
				len = MAX_QUESTION_LEN - strnlen(c->question, MAX_QUESTION_LEN);
				strncat(c->question, line, len);
			}
		}
		
		// answer block starts
		if (amode) {
			qmode=0;
			if ((line[0]=='-') || (line[0]=='+')) {
				len = MAX_ANSWER_LEN - strnlen(c->answers[c->ans_count], MAX_ANSWER_LEN);
				strncat(c->answers[c->ans_count++], line, len);
				ac++;
				if (c->ans_count>MAX_ANSWERS-1) {
					fprintf(stderr, "Too many answers! (Maximum: %i)\n", MAX_ANSWERS);
					ret=1; goto here;
				}
			} else { // Continuation lines for that answer possible
				if (c->ans_count==0) { // First answer MUST start with either '-' or '+' 
					fprintf(stderr, "ERROR: Invalid answer section in file '%s' line %i!\n",cf,i);
					ret=1; goto here;
				}
				len = MAX_ANSWER_LEN - strnlen(c->answers[c->ans_count-1], MAX_ANSWER_LEN);
				strncat(c->answers[c->ans_count-1], line, len);
				// NOTE: Don't increment c->ans_count here because this line is only a continuation!
			}
		}
	}

	// above algorithm creates one unneeded list entry at the end; remove it
	if (qc>0) remove_clist_entry(c_head->prev);
		
	if (verbose) fprintf(stderr, " Read %i lines, %i questions and a total of %i answers.\n", i, qc, ac);
	here:
	if (fp!=NULL) fclose (fp);
	
	return ret;
}


// returns maximum used value in list
// or 0 if all (remaining) elements have same used value
int get_max_used()
{
	struct collection_struct *cur = c_head;
	int max=0, min=0xffffff;
	
	while (cur->next != c_head) {
		cur=cur->next;
		if (cur->used > max) max=cur->used;
		if (cur->used < min) min=cur->used;
	}
	
	if (min==max) max=0;
	return max;
}



// Idea:
// 1. Determine how many list entries to kick out (N = clist_items - q)
// 2. Iterate over list N times and always eliminate element with highest used value
// 3. If highest used value is zero than select q questions randomly
// 
// RETURN 1 upon failure, 0 upon success
int select_questions (int q)
{
	struct collection_struct *cur = c_head;
	int i,j, k, m, N = clist_items-q;
	
	if (N<0) {
		fprintf(stderr, "ERROR: Not enough questions available!\n");
		return 1;
	}
	
	if (verbose) {
		fprintf(stderr, " Selecting %i questions... ", q);
		fflush(stderr);
	}
	
	for (i=0; i<N; i++) {
		m = get_max_used();
		if (m==0) { 
			//
			// Evenly distributed: Eliminate remaining N=clist_items-q randomly
			// 
			N = clist_items-q; // re-calculate actual value of N
			if (N<0) { // should NEVER happen!
				fprintf(stderr, "ERROR: Not enough questions available!\n");
				return 1;
			}
			if (verbose) fprintf(stderr, "(chosen randomly)\n");
			for (j=0; j<N; j++) {
				k=(int) ( ((float) rand()/RAND_MAX)*clist_items); // => kill k'th list entry
				cur=c_head;
				while (cur->next != c_head) {
					cur=cur->next;
					k--;
					if (k<=0) {
						if (verbose==2) fprintf(stderr, " eliminate %08x\n", cur->hash);
						remove_clist_entry(cur);
						break;
					}
				}
			}
			goto done;
		} else { 
			//
			// continue to eliminate
			// 
			cur=c_head;
			while (cur->next != c_head) {
				cur=cur->next;
				if (cur->used == m) {
					if (verbose==2) fprintf(stderr, " eliminate %08x\n", cur->hash);
					remove_clist_entry(cur);
					break;
				}
			}
			
		}
	}
	
	done: 
	
	return 0;
}

