/*
 * EXAMINATOR -- Create your multiple choice exams on the fly.
 * Copyright (C) 2010 Herbert Haas
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, see http://www.gnu.org/licenses/gpl-2.0.html
 *
 */

#include "examinator.h"


	    
int main(int argc, char *argv[])
{

	FILE *conf=NULL, *exam=NULL, *stats=NULL, *solutions=NULL;
	char 
		line[MAX_LINE_LEN], 
		str[MAX_TOKEN_LEN],
		str2[MAX_TOKEN_LEN],
		group_str[MAX_TOKEN_LEN],
		t[5][MAX_TOKEN_LEN], 
		cf[MAX_LINE_LEN];
	char *tmp;
	int
		i,
		n, 
		q, 
		cover_exists=0, 
		msg_flag=0, 
		path_flag=0,
		preamble_flag=0,
		l;
	time_t mytime;

	if (verbose) {
		fprintf(stderr, "EXAMINATOR version %s.\n", VERSION);
		fprintf(stderr, "%s\n", COPYRIGHT);
	}

	verbose = 0;
	configfile[0]='\0';
	line[0]='.'; line[1]='\0';
	
	// ----- Initialize list head element
	c_head = &c_head_element;
	c_head->id=-1;               // = "not to be used"
	c_head->next=c_head; 
	c_head->prev=c_head;
	c_head->question[0] = '\0';
	for (i=0; i<MAX_ANSWERS; i++) c_head->answers[i][0]='\0';
	c_head->used=0;
	c_head->ans_count=0;
	c_head->hash=0;

	clist_items=0; // global count of clist items (head element does NOT count!)
	statistics_entries=0;
	qword[0]='\0';
	group[0]='\0';
	group_str[0]='\0';
	exam_date[0]='\0';
	head_title[0]='\0';
	line_width[0]='\0';
	
	// ----- Initialize random generator
	time(&mytime);
	srand((unsigned int)mytime);
	
	// ----- parse arguments
	if (getopts(argc, argv)) { fprintf(stderr, USAGE); goto fin; }

	if (group[0]!=0x00) {
		fprintf(stderr, "Prepare an exam for '%s'\n", group);
		safe_strncpy(group_str, group, 64);
		for (i=0; i<63; i++) if (group_str[i]==' ') group_str[i]='_';
	}
	
	// ----- open configuration file
	conf = fopen(configfile, "r");
	if (conf==NULL) {
		fprintf(stderr, "No exam configuration file '%s' found.\n", configfile);
		goto fin;
	}

	// ----- generate other filenames (tex and stats) based on configfilename
	tmp = strtok(configfile, ".");
	safe_strncpy(str, tmp, MAX_TOKEN_LEN-5);
	safe_strncpy(str2, tmp, MAX_TOKEN_LEN-15);
	strncat(line, str, MAX_TOKEN_LEN-5);
	strncat(line, ".stats", MAX_TOKEN_LEN-5);
	if (group_str[0]!='\0') {
		strcat(str, "_");
		strcat(str, group_str);
		strcat(str2, "_");
		strcat(str2, group_str);
	}
	strcat(str, ".tex");
	strcat(str2, "_solutions.tex");
	if (verbose) {
		fprintf(stderr, "Statistics are stored in file '%s'.\n", line);
		fprintf(stderr, "Creating exam file '%s'...\n", str);
	}

	// ----- Open statistic file
	stats = fopen (line, "r+");
	if (stats==NULL) {
		stats = fopen (line, "w+");
		if (stats==NULL) {
			fprintf(stderr, "Cannot access statistics file '%s'.\n", line);
			goto fin;
		}
	}
	
	// ----- Open LaTeX file (aka 'examfile' here) 
	exam = fopen(str, "w+");
	if (exam==NULL) {
		fprintf(stderr, "Cannot create exam file '%s'.\n", str);
		goto fin;
	}

	solutions = fopen(str2, "w+");
	if (solutions==NULL) {
		fprintf(stderr, "Cannot create solutions file '%s'.\n", str2);
		goto fin;
	}

	// ----- Read statistics if available
	read_statistics(stats);
	Q=0; // we need that in latex.c
	
	// ----- Parse config file
	while (fgets(line, MAX_LINE_LEN-1, conf) != NULL) {
		n = sscanf(line, "%s %s %s %s %s", t[0], t[1], t[2], t[3], t[4]);
		// skip comment lines
		if ((n>0) && (str_is_comment(t[0]))) continue;
		if ((n==3) && (strncasecmp(t[0], "cover", 9)==0) && (strncmp(t[1], "=", 1))==0) {
			if (cover_exists) {
				fprintf(stderr, "Error: Only a single coverpage file allowed!\n");
				fprintf(stderr, "       *** Edit '%s' and make corrections.\n", configfile);
				goto fin;
			}		      
			if (verbose) fprintf(stderr, "Using coverpage file '%s'...\n", t[2]);
			safe_strncpy(cover_page, t[2], 62);
			cover_exists=1;
		}
		
		if ((n==3) && (strncasecmp(t[0], "qword", 5)==0) && (strncmp(t[1], "=", 1))==0) {
			l=strnlen(t[2], 64);
			if (l>=(63)) { 
				fprintf(stderr, "Error: 'qname' must be smaller than 63 characters\n");
				goto fin;
			}
			safe_strncpy(qword, t[2], 62);
		}
		
		if ((n==3) && (strncasecmp(t[0], "line", 5)==0) && (strncmp(t[1], "=", 1))==0) {
			l=strnlen(t[2], 8);
			if (l>=(8)) { 
				fprintf(stderr, "Error: 'line' must be smaller than 8 characters\n");
				goto fin;
			}
			safe_strncpy(line_width, t[2], 8);
		}

		if ((n>2) && (strncasecmp(t[0], "date", 5)==0) && (strncmp(t[1], "=", 1))==0) {
			(void) strtok(line, "=\n");
			tmp = strtok(NULL, "=\n"); 
			if (tmp==NULL) {
				fprintf(stderr, "Error: 'date' empty?\n");
				goto fin;
			}
			if (strnlen(tmp, MAX_TOKEN_LEN)>(MAX_TOKEN_LEN-1)){
				fprintf(stderr, "Error: 'date' must be smaller than %i characters\n", MAX_TOKEN_LEN);
				goto fin;
			}
			safe_strncpy(exam_date, tmp, MAX_TOKEN_LEN);
			if (verbose==2) fprintf(stderr, "Using date '%s'\n", exam_date);
		}

		if ((n>2) && (strncasecmp(t[0], "head", 5)==0) && (strncmp(t[1], "=", 1))==0) {
			(void) strtok(line, "=\n");
			tmp = strtok(NULL, "=\n"); 
			if (tmp==NULL) {
				fprintf(stderr, "Error: 'head' empty?\n");
				goto fin;
			}
			if (strnlen(tmp, MAX_TOKEN_LEN)>(MAX_TOKEN_LEN-1)){
				fprintf(stderr, "Error: 'head' must be smaller than %i characters\n", MAX_TOKEN_LEN);
				goto fin;
			}
			safe_strncpy(head_title, tmp, MAX_TOKEN_LEN);
			if (verbose) fprintf(stderr, "Using headline '%s'\n", head_title);
		}

		
		if ((n==3) && (strncasecmp(t[0], "path", 5)==0) && (strncmp(t[1], "=", 1))==0) {
			l=strnlen(t[2], MAX_TOKEN_LEN);
			if (l>=(MAX_TOKEN_LEN-2)) { 
				fprintf(stderr, "Error: 'path' must be smaller than %i characters\n", MAX_TOKEN_LEN-2);
				goto fin;
			}

			safe_strncpy(path, t[2], MAX_TOKEN_LEN-1);
			if (t[2][l-1]!='/') strncat(path, "/", 1);
			path_flag=1;
			if (verbose) fprintf(stderr, "Assume the repository is in '%s'...\n", path);
		}


		
		if ((n==3) && (strncasecmp(t[0], "include", 9)==0)) {
			// ----- Add LaTeX preamble -- do this only once!
			if (!preamble_flag) {
				add_latex_preamble(exam);
				add_latex_preamble(solutions);
				preamble_flag=1;
				if (cover_exists) add_latex_coverpage(exam, cover_page);
			}
			
			if ((cover_exists==0) && (msg_flag==0)) {
				fprintf(stderr, "NOTE: No coverpage file specified in '%s'.\n", configfile);
				msg_flag=1;
			}
			if (path_flag) {
				safe_strncpy(cf, path, MAX_TOKEN_LEN);
			} else cf[0]='\0';
			strncat(cf, t[1], MAX_TOKEN_LEN);
			if (add_collection(cf)==1) goto fin;
			create_hashes(); // add hashes to collection
			stats_to_list(); // add statistic data (from stats file) to list
			if (verbose==2) dump_clist();
			q = (int) str2int(t[2]);
			if (select_questions(q)==1) goto fin;
			if (verbose==2) dump_clist();
			print_questions(exam, solutions);
			list_to_stats();
			delete_clist (); 
		}
		
	}

	fin: // ----- Close files and remove clist
	
	write_statistics(stats);

	if (verbose) { fprintf(stderr, "Cleanup and closing all files... "); fflush(stderr); }
	if (conf!=NULL) fclose(conf);
	if (exam!=NULL) {
		end_latex_document(exam);
		end_latex_document(solutions);
		fclose(exam);
		}
	if (stats!=NULL) fclose(stats);
	if (solutions!=NULL) fclose(solutions);
	delete_clist ();
	if (verbose) fprintf(stderr, "finished. Thank you for using EXAMINATOR.\n");
	return 0;
}
