/*
 * EXAMINATOR -- Create your multiple choice exams on the fly.
 * Copyright (C) 2010 Herbert Haas
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, see http://www.gnu.org/licenses/gpl-2.0.html
 *
 */

#ifndef EXAMINATOR
#define EXAMINATOR

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <time.h>


#define VERSION    "0.2"
#define COPYRIGHT  "(C) 2010 by Herbert Haas. See http://www.perihel.at/examinator/ for latest news."
#define MAX_QUESTIONS 5000          // how many questions are supported for statistics
#define MAX_LINE_LEN 512
#define MAX_TOKEN_LEN 128
#define MAX_QUESTION_LEN 2000       // NOTE: must be a multiple of 8
#define MAX_ANSWERS 20              // how many answers per question at max
#define MAX_ANSWER_LEN 512
#define DEFAULT_PICTURE_SIZE 50     // percent of textwidth
#define USAGE "Usage: examinator [options] examfile.conf\n"


int verbose;

char configfile[MAX_LINE_LEN];

struct collection_struct {
	struct collection_struct *next;
	struct collection_struct *prev;
	int id;
	char question[MAX_QUESTION_LEN];
	char picture[MAX_TOKEN_LEN];
	int picture_size;
	char answers[MAX_ANSWERS][MAX_ANSWER_LEN];
	int ans_count;
	u_int32_t hash;
	int used;
} c_head_element;

struct collection_struct * c_head;
int clist_items;
int Q;
char qword[64];
char group[64];
char path[MAX_TOKEN_LEN];
char head_title[MAX_TOKEN_LEN];
char exam_date[MAX_TOKEN_LEN];
char cover_page[MAX_TOKEN_LEN];
char line_width[8];

// For statistics I don't maintain another linked list,
// since even 5000 questions leads to a 60 kbyte array only.
struct stats_struct {
	u_int32_t hash;
	int id;
	int used;
} statistics[MAX_QUESTIONS]; 
int statistics_entries;


int add_collection (char* cf);
int select_questions (int q);
char * safe_strncpy(char *dest, const char *src, size_t n);
unsigned long int str2int(char *str);
int str_is_comment (char *str);
int getopts (int argc, char *argv[]);
int add_latex_preamble(FILE* fp);
int add_latex_coverpage(FILE* fp, char *fname);
int end_latex_document(FILE* fp);

int line_is_empty (char *str);
int clist_set_default (struct collection_struct *cur);
struct collection_struct * create_clist_entry();
int dump_clist ();
struct collection_struct * remove_clist_entry (struct collection_struct * cur);
void delete_clist ();
unsigned long int xstr2int(char *str);
int read_statistics (FILE *stats);
u_int32_t hash(char *str);
int create_hashes();
int stats_to_list();
int get_max_used();
int print_questions(FILE *fp, FILE *sp);
int write_statistics (FILE *stats);
int list_to_stats ();

#endif
