/*
 * EXAMINATOR -- Create your multiple choice exams on the fly.
 * Copyright (C) 2010 Herbert Haas
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, see http://www.gnu.org/licenses/gpl-2.0.html
 *
 */

#include "examinator.h"

int add_latex_preamble(FILE* fp)
{
	fprintf(fp, 
		"\\documentclass[11pt]{scrartcl} \n"
		"\\usepackage[utf8]{inputenc} \n"
		"\\usepackage[T1]{fontenc} \n"
		"\\usepackage[ngerman]{babel}\n"
		"\\usepackage{amsmath}\n"
		"\\usepackage{boxedminipage}\n"
		"\\usepackage{latexsym}\n"
		"\\usepackage{graphicx}\n"
		"\\usepackage{fancyhdr}\n"
                "\\newcommand{\\BF}[1]{\\textbf{#1}}\n"
		"\\newcommand{\\EM}[1]{\\emph{#1}}\n"
		"\\newcommand{\\TT}[1]{\\texttt{#1}}\n"
		"\\usepackage{geometry}\\geometry{body={18cm,22cm},top=3cm,left=1.5cm}\n"
		"\\newcommand{\\bild}[2]{\\centering\\includegraphics[width=#1\\textwidth]{#2}}\n"
		"\\pagestyle{fancy}\\fancyhf{}\n"
		"\\lhead{%s}\n"
		"\\chead{%s}\n"
		"\\rhead{%s}\n"
		"\\lfoot{\\tiny{Created with eXaminator v%s\\\\http:/{\\kern-2pt}/www.perihel.at/examinator}}\n"
		"\\rfoot{\\thepage}\n"
		"\\newenvironment{extemize} {\n"
		"\\begin{itemize}\n"
		"\\renewcommand{\\labelitemi}{$\\Box$}}\n"
		"{ \\end{itemize} }\n"
		"\\begin{document}\n"
		"%% --------------------------\n",
		head_title, group, exam_date, VERSION);
	return 0;
}


// Only copies all text from fname to fp
int add_latex_coverpage(FILE* fp, char *fname)
{
	FILE *p;
	char line[MAX_LINE_LEN];
	
	p = fopen(fname, "r");
	if (p==NULL) return 1;

	while (fgets(line, MAX_LINE_LEN-1, p)!=NULL) {
		fprintf(fp, "%s", line);
	}
	fclose(p); 
	return 0;
}


int print_questions(FILE *fp, FILE *sp)
{
	struct collection_struct *cur = c_head;
	int i;
	if ((fp==NULL)||(sp==NULL)) return 1;
	
	while (cur->next != c_head) {
		cur=cur->next;
		Q++;
		fprintf(fp, "\\begin{minipage}[tl]{150mm}\n");
		fprintf(sp, "\\begin{minipage}[tl]{150mm}\n");
		fprintf(sp, "\\textbf{%s %i:} %s\n", (qword[0]=='\0') ? "" : qword, Q, cur->question);
		fprintf(sp, "\\begin{itemize}\n");
		if (qword[0]=='\0')
			fprintf(fp, "\\begin{enumerate}\\setcounter{enumi}{%i} \\item %s", Q-1, cur->question);
		else
			fprintf(fp, "\\textbf{%s %i:} %s", qword, Q, cur->question);
		fprintf(fp, "\\begin{extemize}\n");
		for (i=0; i<cur->ans_count; i++) {
			fprintf(fp, "\\item %s", &cur->answers[i][1]);
			if (cur->answers[i][0]=='+') fprintf(sp, "\\item %s", &cur->answers[i][1]);
		}
		fprintf(fp, "\\end{extemize}\n");
		if (qword[0]=='\0') fprintf(fp, "\\end{enumerate}\n");
		if (cur->picture_size) fprintf(fp, "\\bild{%f}{%s%s}\n", (double) cur->picture_size/100, path, cur->picture);
		if (line_width[0]!='\0') fprintf(fp, "\\rule{\\textwidth}{%smm}\n", line_width);
		fprintf(fp, "\\end{minipage}\n");
		fprintf(fp, "\\vspace{10mm}\n\n");
		fprintf(sp, "\\end{itemize}\n");
		fprintf(sp, "\\end{minipage}\n");
		fprintf(sp, "\\vspace{10mm}\n\n");
	}
	return 0;
}



int end_latex_document(FILE* fp)
{
	fprintf(fp, "\\end{document}\n");
	return 0;
}


