/*
 * EXAMINATOR -- Create your multiple choice exams on the fly.
 * Copyright (C) 2010 Herbert Haas
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, see http://www.gnu.org/licenses/gpl-2.0.html
 *
 */

#include "examinator.h"

// Sets given list element to default values
int clist_set_default (struct collection_struct *cur)
{
	int i;
	
	if (cur==NULL) return 1;
	
	if (cur->prev != cur) cur->id = cur->prev->id +1;
	cur->question[0]='\0';
	for (i=0; i<MAX_ANSWERS; i++) cur->answers[i][0]='\0';
	cur->used=0;
	cur->hash=0;
	cur->ans_count=0;
	cur->picture_size=0; // means: no picture!
	
	return 0;
}


// Returns pointer to new list element or NULL upon problem
struct collection_struct * create_clist_entry()
{
	struct collection_struct *cur = c_head;
	struct collection_struct *prev; 

	// find end of current list
	while (cur->next != c_head) cur=cur->next;

	prev = cur;
	
	cur = (struct collection_struct*) malloc (sizeof(struct collection_struct));
	if (cur==NULL) return NULL;
	
	prev->next = cur;
	c_head->prev = cur;
	cur->next = c_head;
	cur->prev = prev;

	clist_set_default (cur);
	clist_items++; // global count of items
	return cur;
}


int dump_clist ()
{
	struct collection_struct *cur = c_head;
	int i;
	
	while (cur->next != c_head) {
		cur=cur->next;
		fprintf(stderr, "Element [%03i] %08x used:%i\n", cur->id, cur->hash, cur->used);
		fprintf(stderr, "QUESTION:\n");
		fprintf(stderr, "%s", cur->question);
		fprintf(stderr, "ANSWERS:\n");
		for (i=0; i<cur->ans_count; i++) fprintf(stderr, "%s", cur->answers[i]);
		fprintf(stderr, "\n");
	}
	return 0;
}



// Removes element 'cur' in the doubly linked list
// Returns pointer to next element
struct collection_struct * remove_clist_entry (struct collection_struct * cur)
{
	struct collection_struct *prev = cur->prev;
	struct collection_struct *next = cur->next;
	
	if ((cur!=NULL) && (cur!=c_head)) {
		free(cur);
		clist_items--;
		prev->next = next;
		next->prev = prev;
	} else return c_head;
	return next;
}


// Removes all but the head element
void delete_clist ()
{
	struct collection_struct *next = c_head->next;
	while (next != c_head) 
		next= remove_clist_entry(next);
	
	clist_items=0; // head element does not count!
}


int create_hashes()
{
	struct collection_struct *cur = c_head;
	
	while (cur->next != c_head) {
		cur=cur->next;
		cur->hash = hash(cur->question);
	}
	return 0;
}


