/*
 * EXAMINATOR -- Create your multiple choice exams on the fly.
 * Copyright (C) 2010 Herbert Haas
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, see http://www.gnu.org/licenses/gpl-2.0.html
 *
 */

#include "examinator.h"


// Reads statistcs file entries into statistcs[].
// 
int read_statistics (FILE *stats)
{
	char line[MAX_LINE_LEN], t[2][MAX_TOKEN_LEN];
	int i=0, l=1, k;
	
	if (verbose) { 
		fprintf(stderr, "Reading statistics... ");
		fflush(stderr);
	}
	if (verbose==2) fprintf(stderr, "\n");
	
	while (fgets(line, MAX_LINE_LEN-1, stats) != NULL) {
		k = sscanf(line, "%s %s", t[0], t[1]);
		if (k==2) {
			statistics[i].hash = (u_int32_t) xstr2int(t[0]);
			statistics[i].used = (int) str2int(t[1]);
			if (verbose==2) fprintf(stderr, " %08x %i\n", statistics[i].hash, statistics[i].used);
			i++;
		} else if (strnlen(line, 10)>0) {
			fprintf(stderr, "Invalid statistics line %i.\n", l);
		}
		l++; // line counter
	}
	statistics_entries = i;
	if (verbose) fprintf(stderr, "%i entries found.\n", i);
	
	return 0;
}



// Assuming that clist only consists of selected (printed) questions.
// Now update statistics array 
// 
int list_to_stats ()
{
	struct collection_struct *cur = c_head;
	int i, found;

	// first update statistics table
	while (cur->next != c_head) {
		cur=cur->next;
		found=0;
		for (i=0; i<statistics_entries; i++) {
			if (statistics[i].hash == cur->hash) {
				statistics[i].used++;
				if (verbose==2) fprintf(stderr, "  Update %08x used %i\n", 
							statistics[i].hash, statistics[i].used);
				found=1;
				break;
			}
		}
		if (!found) {
			statistics[statistics_entries].hash = cur->hash;
			statistics[statistics_entries].used = cur->used+1;
			statistics_entries++;
		}
	}
	
	return 0;
}


// Add count information to list.
// This method is efficient enough for <5000 questions I think
int stats_to_list()
{
	struct collection_struct *cur;
	int i;

	if (verbose) {
		fprintf(stderr, "Updating statistics to list... ");
		fflush(stderr);
	}
	for (i=0; i<statistics_entries; i++) {
		cur = c_head;
		while (cur->next != c_head) {
			cur=cur->next;
			if (statistics[i].hash==cur->hash) {
				cur->used = statistics[i].used;
				break;
			}
		}
	}
	if (verbose) fprintf(stderr, "done.\n");
	return 0;
}




// Write statistics table to file
int write_statistics (FILE *stats)
{
	int i;
	if (stats==NULL) return 1;
	rewind(stats);
	for (i=0; i<statistics_entries; i++) {
		fprintf(stats, "%08x %i\n", statistics[i].hash, statistics[i].used);
	}
	return 0;
}