/*
 * EXAMINATOR -- Create your multiple choice exams on the fly.
 * Copyright (C) 2010 Herbert Haas
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, see http://www.gnu.org/licenses/gpl-2.0.html
 *
 */

#include "examinator.h"


// Behaves much like strncpy but additionally ensures that dest is *ALWAYS*
// \0-terminated. 
char * safe_strncpy(char *dest, const char *src, size_t n)
{
	char *tmp;
	tmp = strncpy(dest, src, n);
	dest[n-1]='\0';
	return tmp;
}

// Convert str to (unsigned long) int
unsigned long int str2int(char *str)
{
	unsigned long int i = strtoul(str, (char **)NULL, 10);
	errno=0;
	if ((errno == ERANGE && (i == ULONG_MAX)) || (errno != 0 && i == 0)) 
		perror("strtoul");
	return i;
}


// Convert hex-str to (unsigned long) int
// Return value: the unsigned long int
unsigned long int xstr2int(char *str)
{
	unsigned long int i;
	errno=0;
	i = strtoul(str, (char **)NULL, 16);
	if ((errno == ERANGE && (i == ULONG_MAX))
	    || (errno != 0 && i == 0))
	{
		perror("strtoul");
	}
	return i;
}



// Returns 1 if str starts with an "!" (=a comment) but ignores preceeding 
// spaces. Return 0 otherwise (=not a comment).
int str_is_comment (char *str)
{
	int i, l=strnlen(str, MAX_TOKEN_LEN);
	for (i=0; i<l; i++)
		if (!isspace(str[i])) {
		    if (str[i]=='!') return 1;
		    else return 0; 
		}
	return 0;
}


// RETURNS 1 if line is empty or contains only of non-printable characters 
int line_is_empty (char *str)
{
	int i=0, len;

	
	len = strnlen (str, MAX_LINE_LEN);
	if (len==MAX_LINE_LEN) { 
		fprintf(stderr, "Warning: Line too long! (Maximum: %i)\n", MAX_LINE_LEN);
		return 0;
	} else 	if (len==0) {
		fprintf(stderr, "Empty line\n");
		return 1;
	} else {
		for (i=0; i<len; i++) {
			if (!isspace(str[i])) return 0;
		}
	}
	return 1;
}

u_int32_t hash(char *str)
{
	char frame[MAX_QUESTION_LEN+1]; 
	int i, len=strnlen(str, MAX_QUESTION_LEN);
	u_int64_t s=0, *sp;
	
	memset((void*) frame, 0x00, MAX_QUESTION_LEN);
	memcpy((void*) frame, str, len);
	
	for (i=0; i<(MAX_QUESTION_LEN/8); i++) {
		sp = (u_int64_t*) &frame[i*8];
		s += *sp;
	}
	s=s%4294967291; // prime
	return (u_int32_t) s;
}


